/*
 Name:		acc_odr_test.ino
 Created:	3/23/2021 10:31:05 AM
 Author:	siver
*/

// the setup function runs once when you press reset or power the board
#include <Adafruit_SPIDevice.h>
#include <Adafruit_I2CRegister.h>
#include <Adafruit_I2CDevice.h>
#include <Adafruit_BusIO_Register.h>
#include <ICM_20948.h>
#include <bmp3_defs.h>
#include <bmp3.h>
#include <Adafruit_BMP3XX.h>
#include <Arduino.h>

//#define DEBUG
#define WIRE_PORT Wire
#define AD0_VAL 1 
#define KNAPP 3
#define SEALEVELPRESSURE_HPA (1013.25)

ICM_20948_I2C myICM;
Adafruit_BMP3XX bmp;

const unsigned long LOGGTID_S = 12;
unsigned long tid_start_logging;
static int cnt = 0, cntP = 0, knapp = 1;
bool start_logging = false;
static int aX, aY, aZ, len;
static char buffer[64]; // Strrelsen p transmission buffer til Arduino UNO Rev.3 / ATmega328p

void setup() {
	Serial.begin(500000);

	WIRE_PORT.begin();
	WIRE_PORT.setClock(400000);

	// Setup SEN-15335
	bool initialized = false;
	while (!initialized) {
		myICM.begin_acc_only(WIRE_PORT, AD0_VAL);

		Serial.print(F("Initialization of the sensor returned: "));
		Serial.println(myICM.statusString());

		// nsker  bruke lavpassfilter for akselerometer.
		// Bruker standardverdier som er definert i funksjonen
		// startupDefault
		myICM.enableDLPF(ICM_20948_Internal_Acc, true);
		
#ifdef DEBUG
		// Gyro avsltt:
		// Sjekk at gyro er av sidan det ikkje er i bruk
		// Datablad: Kap. 8.5 PWR_MGMT_2 og Kap.7.1 User Bank 0 Register Map
		uint8_t val;
		myICM.setBank(0);
		myICM.read(AGB0_REG_PWR_MGMT_2, &val, 1);
		Serial.print("Disable gyro: ");
		Serial.println(val, BIN);

		// Mleomrde:
		// Datablad: Kap. 10.15 ACCEL_CONFIG og Kap. 7.3 User Bank 2 Register Map
		myICM.setBank(2);
		myICM.read(AGB2_REG_ACCEL_CONFIG, &val, 1);
		Serial.print("Accel Full-Scale: ");
		Serial.println(val, BIN);

		// Samplingsrate:
		// Datablad: Kap. 7.3 User Bank 2 Register Map og Kap. 10.11/12 ACCEL_SMPLRT_DIV_1/2
		uint8_t div1, div2;
		uint16_t SMPLRT_DIV;
		myICM.setBank(2);
		myICM.read(AGB2_REG_ACCEL_SMPLRT_DIV_1, &div1, 1);
		myICM.read(AGB2_REG_ACCEL_SMPLRT_DIV_2, &div2, 1);
		SMPLRT_DIV = (div1 << 8) | div2;

		Serial.print("Samplerate register 1: ");
		Serial.print(div1, BIN);
		Serial.print("\tReg2: ");
		Serial.println(div2, BIN);
		Serial.print("Samplerate [Hz]: ");
		Serial.println(1125 / (1 + SMPLRT_DIV));
#endif // DEBUG

		if (myICM.status != ICM_20948_Stat_Ok) {
			Serial.println("Trying again...");
			delay(500);
		}
		else initialized = true;
	}

	// SETUP BMP390
	if (!bmp.begin_I2C()) {
		Serial.println("Could not find a valid BMP3 sensor, check wiring!");
		while (true);
	}

	//Set up oversampling and filter initialization
	// Datablad: Kap. 3.5 Filter Selection og Kap. 3.9 Measurement Timings
	// Sensoren er i Forced Mode, instilling for ODR gjeld ikkje d.
	// Viktig  overhalde maks. samplingsrate oppgitt i kap. 3.9

#ifdef DEBUG
	Serial.print("Klarte a sette TEMP_OS til valgt verdi: ");
	Serial.println(bmp.setTemperatureOversampling(BMP3_OVERSAMPLING_2X));
	Serial.print("Klarte a sette PRES_OS til valgt verdi: ");
	Serial.println(bmp.setPressureOversampling(BMP3_OVERSAMPLING_4X));
	Serial.print("Klarte a sette IIR-filter til valgt verdi: ");
	Serial.println(bmp.setIIRFilterCoeff(BMP3_IIR_FILTER_COEFF_15));
	Serial.print("Klarte a sette Output Datarate til valgt verdi (ikkje gyldig i Forced Mode): ");
	Serial.println(bmp.setOutputDataRate(BMP3_ODR_100_HZ));
	Serial.print("Sjekk at innstillinger er gyldige. 0 = alt OK: ");
	Serial.println(bmp.changeSettings());
#else	
	bmp.setTemperatureOversampling(BMP3_OVERSAMPLING_2X);
	bmp.setPressureOversampling(BMP3_OVERSAMPLING_4X);
	bmp.setIIRFilterCoeff(BMP3_IIR_FILTER_COEFF_15);
	bmp.setOutputDataRate(BMP3_ODR_100_HZ);
	bmp.changeSettings();
#endif // DEBUG
}

// the loop function runs over and over again until power down or reset
void loop() {
	// Antar at Arduino fr ein tekststreng liknande "START\n"
	// for  starte logging
	if (Serial.available()) {
		String s = Serial.readStringUntil('\n');
		if (s.length() > 0) {
			start_logging = !start_logging;
			cnt = 0;
			cntP = 0;
			tid_start_logging = millis();

			// Utfrer ei mling av trykk for  unng udefinert trykk ved start
			bmp.performReading();
			cntP++;
		}
	}


	while (start_logging) {
		// Avlesing av akselerasjon
		myICM.getAcc();
		cnt++;

		aX = myICM.accX();
		aY = myICM.accY();
		aZ = myICM.accZ();

		// Trykk blir kun mlt ein gong for kvar sjuande mling av akselerasjon
		// Dette er for  ikkje redusere samplingsraten til akselerometeret for mykje
		if (cnt % 7 == 0) {
			bmp.performReading();
			cntP++;
		}
		knapp = digitalRead(KNAPP);

		// Utskrift av mlinger
		len = sprintf(buffer, "%i, %i, %i, ", aX, aY, aZ);
		for (int i = 0; i < len; i++)
		{			
			Serial.write(buffer[i]);
		}
		Serial.print(bmp.pressure);
		len = sprintf(buffer, ", %i\n", knapp);
		for (int i = 0; i < len; i++)
		{
			Serial.write(buffer[i]);
		}
		if (millis() > tid_start_logging + (LOGGTID_S * 1000)) {
			start_logging = false;
			Serial.println("Avslutter logging.");
#ifdef DEBUG
			Serial.print("Antall lesinger acc: ");
			Serial.println(cnt);
			Serial.print("Antall lesinger trykk: ");
			Serial.println(cntP);
			Serial.print("Frekvens: ");
			Serial.println(cnt / LOGGTID_S);
#endif // DEBUG
		}
	}
}